# One-way repeated measures ANOVA
# by Aaron Albin
# Script began: 4/1/2012

###########################
# WHEN TO USE THIS SCRIPT #
########################### 

# Use this script if all of the following is true:
# * You have measurements that you wish to compare between one set of different non-numeric 'conditions'.
# * Multiple individual 'subjects' (e.g. participants in an experiment) were measured over the course of data collection, and you have multiple measurements from each individual subject.
# * Your data are stored in three separate columns - one with the measurements, another with the labels for the 'conditions', and a third with some sort of identifier that keeps track of which measurement belongs to which individual subject.

# An example: Investigating the relationship between college students' year in school (a set of non-numeric 'conditions') on the number of hours slept every night over the course of a month.

# Note: If you don't have approximately the same number of data points inside each condition (e.g. because of several NA data points), the results must be taken with care, since the statistical procedures underlying R's ANOVA function 'aov()' were designed for 'balanced designs'.

###############
# SELECT DATA #
###############

# Run this to have R tell you the names for each of the columns that it ended up with after importing the data frame:
#=====[1]=====
colnames(Dataframe)
#=============

# Now, pick out which of these columns has your number data for the ANOVA and which has the identifier labels. Type the name of those columns between the double-quotes below.
#=====[2]=====
Name_NumberColumn  = "NumberColumnName"
Name_LabelColumn   = "LabelColumnName"
Name_SubjectColumn = "SubjectColumnName"
# Name_NumberColumn = "VOT1"; Name_LabelColumn = "CONSONAT"; Name_SubjectColumn = "SPKR"
#=============

# Now run this to extract out the specific columns data you selected.
# The function 'as.factor()' ensures that your conditions will be treated as conditions (e.g. not numbers if they are subject numbers (1,2,3,...) etc.)
#=====[3]=====
NumberData = Dataframe[,Name_NumberColumn]
Conditions = as.factor(Dataframe[,Name_LabelColumn  ])
Subjects   = as.factor(Dataframe[,Name_SubjectColumn])
#=============

################
# RUN ANALYSES #
################

# In the output, it gives you a confidence interval. R defaults to a 95% confidence interval, as this is common practice in the literature, but you can adjust the percentage here if you like (to 99%, 99.9% etc.).
# You must specify this in decimal form, i.e. 0.95, 0.99, 0.999, etc.:
#=====[4]=====
PercentageForConfidenceInterval = 0.95
#=============

# Run these five lines of code to conduct the analysis:
#=====[5]=====
FittedModel_FullDataset=aov(NumberData ~ Conditions + Error(Subjects/Conditions), qr=TRUE, contrasts=NULL)
ResultsSummary = summary(object=FittedModel_FullDataset, intercept=FALSE, keep.zero.df=TRUE)
AggregatedData = aggregate(NumberData, by=list(Subjects=Subjects,Conditions=Conditions), FUN=mean)
FittedModel_AggregatedData=aov(x ~ Conditions, qr=TRUE, contrasts=NULL, data=AggregatedData)
ConditionMeans = model.tables(x=FittedModel_AggregatedData,type="means",se=FALSE)
TukeyHSD = TukeyHSD(x=FittedModel_AggregatedData,ordered=FALSE, conf.level=PercentageForConfidenceInterval)
#=============

#######################
# INSPECT THE RESULTS #
#######################

# Run the following code for a reformatted, easy-to-read synthesis of the various results:
#=====[6]=====
{cat("\n\nDESCRIPTIVE STATISTICS\n~~~~~~~~~~~~~~~~~~~~~~\nOverall mean across the entire dataset = ",ConditionMeans$tables$'Grand mean',"\n\nMean for each condition:\n",sep="")
nConditions = length(ConditionMeans$tables$Conditions)
for(EachCondition in 1:nConditions){ExtractedNSet = ConditionMeans$n
if(length(ExtractedNSet)==1){ExtractedN=ExtractedNSet}else{ExtractedN = ExtractedNSet[EachCondition]}
if(!is.na(ExtractedN)){CurrentN=ExtractedN}else{CurrentN=ConditionMeans$n$Conditions[1]}
cat(" * Condition '",names(ConditionMeans$tables$Conditions)[EachCondition],"': ",ConditionMeans$tables$Conditions[EachCondition]," (",CurrentN," observations)\n",sep="")}
cat("\n\nTEST THAT WAS RUN\n~~~~~~~~~~~~~~~~~\nA one-way repeated measures ANOVA was run on '",Name_NumberColumn,"' as grouped by '", Name_LabelColumn, "',\n   with '",Name_SubjectColumn,"' used as the error term for individual subjects.\n\n\nRESULTS\n~~~~~~~\nOverall across all conditions of '", Name_LabelColumn, "' (within subjects):\n   F(",ResultsSummary[[2]][[1]]["Conditions","Df"],",",ResultsSummary[[2]][[1]]["Residuals","Df"],") = ",ResultsSummary[[2]][[1]]["Conditions","F value"],", p = ",ResultsSummary[[2]][[1]]["Conditions","Pr(>F)"],"\n\n",sep="")
nComparisons = nrow(TukeyHSD$Conditions)
for(EachComparison in 1:nComparisons){cat("Comparison '",rownames(TukeyHSD$Conditions)[EachComparison],"': Mean = ",TukeyHSD$Conditions[EachComparison,"diff"],", ", 100*PercentageForConfidenceInterval, "% confidence interval [",TukeyHSD$Conditions[EachComparison,"lwr"],", ",TukeyHSD$Conditions[EachComparison,"upr"],"], p = ",TukeyHSD$Conditions[EachComparison,"p adj"],"\n",sep="")}
if(ncol(ResultsSummary[["Error: Subjects"]][[1]])>4){cat("\nBetween-subjects effect: F(",ResultsSummary[["Error: Subjects"]][[1]]["Conditions","Df"],",",ResultsSummary[["Error: Subjects"]][[1]]["Residuals","Df"],") = ",ResultsSummary[["Error: Subjects"]][[1]]["Conditions","F value"],", p = ",ResultsSummary[["Error: Subjects"]][[1]]["Conditions","Pr(>F)"],"\n\n",sep="")}else{cat("\n\n")}}
#=============

# The following is R's default way of displaying the results of the various tests performed above:
#=====[7]=====
show(ConditionMeans)
show(ResultsSummary)
show(TukeyHSD)
#=============

