# Customizable ANOVA
# by Aaron Albin
# Script began: 2/15/2012

###########################
# WHEN TO USE THIS SCRIPT #
########################### 

# Use this script if all of the following is true:
# * You have measurements that you wish to compare between multiple sets of different non-numeric 'conditions'.
# * You have a clear idea about how the various sets of conditions relate to one another in the way they impact the measurements.
# * Your data are stored in multiple separate columns - one with the measurements and several others with identifier labels.

# An example: Investigating the relationship between college students' year in school, academic major, and in/out-of-state status (different sets of non-numeric 'conditions') on the number of hours slept per night.

# Note: If you don't have approximately the same number of data points inside each combination of conditions (e.g. because of several NA data points), the results must be taken with care, since the statistical procedures underlying R's ANOVA function 'aov()' were designed for 'balanced designs'.

###############
# SELECT DATA #
###############

# Run this to have R tell you the names for each of the columns that it ended up with after importing the data frame:
#=====[1]=====
colnames(Dataframe)
#=============

###################
# SPECIFY FORMULA #
###################

# In this part of the script, you'll be writing a formula expressing how you think your variables are related to one another. It's important you understand how to specify models generally.
# Below, seven examples are given of relationships between a predicted ('response'/'dependent') variable 'y' and one or more predictor ('independent') variables 'a', 'b', and 'c'.
# The specific labels you'll use will come from the column names you obtained from Step 1.
# The syntax in these nine examples can be generalized to cases with 4 or more terms in the model.
# NOTE: The usage of the '+', '-', '*', '^', and ':' symbols in the specification of formulas here is special, diverging from the typical meaning of these symbols in R generally.

# [EXAMPLE 1] Formula = y ~ a
# This describes a simple linear relationship between y and a

# [EXAMPLE 2] Formula = y ~ a + b
# This says that y is a combination of a and b, but there is no interaction term. The '+' is how you combine multiple terms one-by-one. This is how 'random block designs' are coded.

# [EXAMPLE 3] Formula = y ~ a + a:b
# This says that y is a combination of a and an interaction of a and b, but not b alone. This illustrates how ':' can be used to explicitly define an interaction term.

# [EXAMPLE 4] Formula = y ~ a + b + a:b
#             Abbreviation: y ~ a*b
# This is a more typical example where y is described by a, b, and an interaction between a and b. Note the convenient abbreviation to describe this.

# [EXAMPLE 5] Formula = y ~ a + b + c
# This says that y is a linear combination of a, b, and c, but no interactions are included at all. Note this is just a simple expansion of Example 2.

# [EXAMPLE 6] Formula = y ~ a + b + c + a:b + b:c + a:c
#             Abbreviation: y ~ (a+b+c)^2
# This says that y is a linear combination of a, b, and c, as well as their 'second-order interactions', but not their 'third-order interaction' a:b:c. Note the convenient abbreviation.

# [EXAMPLE 7] y ~ a + b + c + a:b + b:c + a:c + a:b:c
#             Abbreviation: y ~ a*b*c
# This is a more typical example where you include a, b, c, and all cominatoric interactions in the model, including a:b:c. Note the very convenient abbreviation, as well as the parallelism with Example 4.

# Finally, any function (including the above seven examples) can also be modified in any of the following ways:
# (1) An individual 'term' (i.e. component of a function) can be removed by putting a '-' symbol in front of it. Thus, for example, 'y ~ a*b - a' is the same as 'y ~ b + a:b'.
# (2) All formulas have an implied intercept term unless it is removed by adding either '-1' or '+0' to the function. Thus, for example, rather than 'y ~ a*b', you would write 'y ~ a*b - 1' or 'y ~ a*b + 0'
# (3) To add a 'subject error term' for examining within-subjects random effects in a Repeated Measures design, for example, rather than y~a*b, use one of the following (depending on specifically which terms in the model you want an error term for).
#       For example, if you have two predictors 'a' and 'b', all of the following are possible:
# y ~ a*b + Error(SubjectID)       # One overall subject term that does not interact with other predictors
# y ~ a*b + Error(SubjectID/(a+b)) # Subject term for main effects only
# y ~ a*b + Error(SubjectID/(a:b)) # Subject term for interaction only
# y ~ a*b + Error(SubjectID/(a*b)) # Subject term for both main effects as well as interaction

# You might see a warning message saying that the 'Error() model is singular', but the analysis will still run.

# If any of your label columns contain numbers (1,2,3,etc.), like subject numbers, then surround just that specific variable name with as.factor().
# For example, if LabelColumn1 were numbers like that, then say: Formula = NumberColumn1 ~ as.factor(LabelColumn1)*LabelColumn2

# Here, please specify your formula:
#=====[2]=====
Formula = NumberColumn1 ~ LabelColumn1*LabelColumn2
# Formula = F1 ~ CONSONAT*SYLLABIF + Error(SPKR/(CONSONAT*SYLLABIF))
#=============

################
# RUN ANALYSES #
################

# In the output, it gives you a confidence interval. R defaults to a 95% confidence interval, as this is common practice in the literature, but you can adjust the percentage here if you like (to 99%, 99.9% etc.).
# You must specify this in decimal form, i.e. 0.95, 0.99, 0.999, etc.:
#=====[3]=====
PercentageForConfidenceInterval = 0.95
#=============

# Run these two lines of code to conduct the analysis:
#=====[4]=====
FittedModel = aov(formula=Formula, data=Dataframe, qr=TRUE, contrasts=NULL) # Fit a linear 'analysis of variance' model to the data using this formula.
ResultsSummary = summary(object=FittedModel,intercept=FALSE,keep.zero.df=TRUE) # Calculate various summary statistics of the model just fitted.
#=============

# If you do *not* have any 'Error()' term in your formula for a repeated measures design, then you can also run the following two lines of code for additional analyses:
# (If you did include an 'Error()' term, then this will issue an error message, so just skip down to step 6.
#=====[5]=====
ConditionMeans = model.tables(x=FittedModel,type="means",se=FALSE) # # Calculate the average of the measurements within each specific condition
TukeyHSD = TukeyHSD(x=FittedModel,ordered=FALSE, conf.level=0.95) # Calculate Tukey's Honest Significant Differences
#=============

#######################
# INSPECT THE RESULTS #
#######################

# The following displays all of the following for each term in the model:
# * "Df" = degrees of freedom
# * "Sum Sq" = sum of squares
# * "Mean Sq" = mean square
# * "F value" = the F statistic from the F-distribution
# * "Pr(>F)" = The p-value (Only given if there are non-zero residual degrees of freedom)
#=====[6]=====
show(ResultsSummary)
#=============

# If you do *not* have any 'Error()' term in your formula for a repeated measures design, then the following will display:
# * basic descriptive statistics, namely the means and Ns for each condition and interaction
# * individual contrasts between all logically possible specific pairings of conditions
# (If you did include an 'Error()' term, then this will issue an error message, so just skip this.)
#=====[7]=====
show(ConditionMeans)
show(TukeyHSD)
#=============
