# Correlation test
# by Aaron Albin
# Script began: 3/12/2012

###########################
# WHEN TO USE THIS SCRIPT #
########################### 

# Use this script if you can answer 'yes' to all of the following questions:

# A) Inside your dataframe, you have two sets of numeric data stored as two separate columns. 
# B) Each row in the dataframe represents one 'observation' in the real world, hence the numbers in each row are inherently 'paired' together.
# C) You are interested in how closely the two sets of numbers are related/associated, i.e. whether an increase in one always mean an increase in the other.
# The correlation value that will emerge out of this analysis is a number from -1 to 1, where -1 indicates a perfectly negative correlation and 1 indicates a perfectly positive correlation.
# The statistical test below tells you the chances that this correlation value is 0, i.e. that there is no association/relation between the two sets of numbers at all.

# Several examples:
# - Is there a relationship between high school SAT scores and college GPA?
# - Is there a relationship between amount of stress and how well you will do on an exam?
# - Is there a relationship between a baby's birth weight and how many colds it will have in infancy?
# Source: http://www.wadsworth.com/psychology_d/special_features/ext/workshops/correlation.html

###############
# SELECT DATA #
###############

# Run this to have R tell you the names for each of the columns that it ended up with:
#=====[1]=====
colnames(Dataframe)
#=============

# Now, pick out which two of these columns have your number data.
# Type the names of those columns between the double-quotes below.
# It doesn't matter which one you call '1' and which you call '2'.
#=====[2]=====
DataName1 = "ColumnName1"
DataName2 = "ColumnName2"
#DataName1="F1";DataName2="F2"
#=============

# Now run this to extract out the specific columns data you selected
#=====[3]=====
NumberData1 = Dataframe[,DataName1]
NumberData2 = Dataframe[,DataName2]
#=============

###########################
# SET UP AND RUN ANALYSIS #
###########################

# Here, specify the 'directionality' of the alternative hypothesis. You have three choices:
# * "two.sided" (Note the period!) - Your alternative hypothesis is that the correlation is NOT EQUAL TO 0.
# * "greater" - Your alternative hypothesis is that the correlation is GREATER THAN 0 (i.e. a positive association).
# * "less" - Your alternative hypothesis is that the correlation is LESS THAN 0 (i.e. a negative association).
#=====[4]=====
Directionality_AlternativeHypothesis = "two.sided"
#=============

# In the output, it gives you a confidence interval (more technically, the asymptotic confidence interval based on Fisher's Z transform)
# R defaults to a 95% confidence interval, as this is common practice in the literature, but you can adjust the percentage here if you like:
#=====[5]=====
PercentageForConfidenceInterval = 0.95
#=============

# Now run the actual 'cor.test()' function. 
# Note that you will not get a confidence interval if you don't have at least 4 complete pairs of observations.
#=====[6]=====
Results = cor.test(x=NumberData1, y=NumberData2, alternative=Directionality_AlternativeHypothesis, method="pearson", conf.level=PercentageForConfidenceInterval)
#=============

#######################
# INSPECT THE RESULTS #
#######################

# Run the following code for a reformatted, easy-to-read representation of the results:
#=====[7]=====
if(Results$alternative=="two.sided"){AlternativeText="not equal to"};if(Results$alternative=="greater"){AlternativeText="greater than "};if(Results$alternative=="less"){AlternativeText="less than"};cat("\n\nDESCRIPTIVE STATISTICS\n~~~~~~~~~~~~~~~~~~~~~~\nThe ",Results$method," between ",DataName1," and ", DataName2," is ", as.numeric(Results$estimate),".\n\nTEST THAT WAS RUN\n~~~~~~~~~~~~~~~~~\nA correlation test was run:\n - Null hypothesis: The true correlation is 0.\n - Alternative hypothesis: The true correlation is ",AlternativeText," 0.\n\nRESULTS\n~~~~~~~\nThe ",100*attributes(Results$conf.int)$conf.level,"% confidence interval ranges from ", as.numeric(Results$conf.int)[1], " to ", as.numeric(Results$conf.int)[2],".\nt(", as.numeric(Results$parameter), ")=", as.numeric(Results$statistic), ", p=",Results$p.value,"\n\n\n",sep="")
#=============

# Here is R's default display of the results:
#=====[8]=====
show(Results)
#=============

#########################
# For advanced users:

# The above test assumes the data follows a t distribution and the samples follow two independent normal distributions (in other words, a bivariate normal distribution).
# If your data are not necessarily distributed in this way, it is recommended you use either of the following instead. They are more robust since they estimate a rank-based measure of association.
# Note that no confidence intervals are given, however.


# The following scales the data to be approximately normal with zero mean and unit variance, then calculates Kendall's tau statistic (with no continuity correction):
#=====[9]=====
cor.test(x=NumberData1, y=NumberData2, alternative=Directionality_AlternativeHypothesis, method="kendall", exact = FALSE, continuity = FALSE)
#=============

# The following uses an Edgeworth series asymptotic t approximation to calculates Spearman's rho statistic (with no continuity correction):
#=====[10]=====
cor.test(x=NumberData1, y=NumberData2, alternative=Directionality_AlternativeHypothesis, method="spearman", exact = FALSE, continuity = FALSE)
#==============
