# Create barplot from one number column and one label column
# by Aaron Albin
# Script began: 4/4/2012

###########################
# WHEN TO USE THIS SCRIPT #
########################### 

# Use this script when you have one single column of number data in your dataframe with a second column containing the corresponding labels (the different categories or conditions).
# You wish to represent the average of all the values in each of these categories or conditions as bars (hence, in a 'barplot').
# You also might want to add 'whiskers' to each of the bars to show the range of dispersion (the 'standard deviation') from this average.

# Note that this looks similar to, but is in fact different from, a histogram.

###############
# SELECT DATA #
###############

# Run this to have R tell you the names for each of the columns that it ended up with after importing the dataframe:
#=====[1]=====
colnames(Dataframe)
#=============

# Now, pick out which of these columns has your number data and which has the identifier labels.
# Type the name of those columns between the double-quotes below (leaving the other code untouched).
#=====[2]=====
NumberData = Dataframe[,"NumberColumn1"]
IdentifierLabels = Dataframe[,"LabelColumn"]
# NumberData = Dataframe[,"K_CONF"]; IdentifierLabels = Dataframe[,"CONSONAT"]
#=============

# Run these two lines to store the names you are using for your identifier labels and also show them to you (so you can double-check that they are as you expect them to be).
#=====[3]=====
BarLabels = levels(IdentifierLabels)
levels(IdentifierLabels)
#=============

# This step is optional - you can skip down to step 5 if you like without causing any problems.
# Below, for each of the labels you just saw in the console from the previous step, you can specify a new label for it using spaces or any special symbols you want. This might make for a more visually-appealing plot.
# If you want to include a line break in the middle of your label, this is written as '\n' (without the single-quotes). For example, "My Experimental\nCondition One".
# Below, we use the [c]ombine function 'c()'. This lets you make a list of as many column names as you want as long as they are separated by commas:
# For example, c("A","B") and c("A","B","C") and c("A","B","C","D") are all fine - you can include as many as you want.
# As you'll notice, for each column-name entry in this list, to make sure R knows we're dealing with names/letters and not numbers, we need to make sure and surround the column name with double-quotes.
# NOTE: It is important that you double-check that you have the same number of entries in this list as the one you just saw in the console from Step 3! Also, keep them in the same order.
#=====[4]=====
BarLabels = c("My first condition", "My second condition", "My third condition")
# BarLabels = c("The [b] sound", "The [p] sound")
#=============

###############################
# SPECIFY PARAMETERS FOR PLOT #
###############################

# Set the name for the main title (centered along the top of the plot) here.
#=====[5]=====
MainTitle = "Barplot"
#=============

# You can have R either create a vertical or horizontal barplot.
# This describes the direction that the bars run on the plot.
# If you make a vertical barplot, your bar labels will be on the bottom of the plot and the numerical scale will be on the left, with bars vertically sticking up. The first bar will be on the left.
# If you make a horizontal barplot, your bar labels will be on the left of the plot and the numerical scale will be on the bottom, with bars horizontally sticking out from the left. The first bar will be on the bottom, which may be opposite of what you'd expect.
# Here, indicate your decision in terms of a TRUE/FALSE decision of 'Should this barplot be horizontal?'
# Remember that TRUE or FALSE must be in all caps and must *not* be surrounded by quotes.
#=====[6]=====
ShouldThisBarplotBeHorizontal = FALSE
#=============

# Generally, it is a good idea to let R determine the limits (i.e. range) of the x (horizontal) axis and y (vertical) axis for you. 
# Leaving the values below as NULL will have this effect.
# However, if you would like to adjust them, you can do that here.
# Just say xAxisLimits or yAxisLimits equal to 'c( ___, ___ )', where the two '___'s indicate the lower and upper bound (respectively) of the range for the relevant axis. Make sure to put the lower bound first (unless you want the axis to be backwards/flip-flopped).
# Note that if ShouldThisBarplotBeHorizontal = FALSE, you will almost always want to keep xAxisLimits as NULL. Similarly, if ShouldThisBarplotBeHorizontal = TRUE, you will almost always want to keep yAxisLimits as NULL.
#=====[7]=====
xAxisLimits = NULL
yAxisLimits = NULL
#=============

# Below the double-quotes below, you can insert a few words you would like to use as the 'axis labels' for the x and y axes.
# If ShouldThisBarplotBeHorizontal = FALSE:
# - xAxisLabel is the superordinate term for whatever all the different bars/categories represent
# - yAxisLabel is the name of the thing being measured, which all your number-data represent
# If ShouldThisBarplotBeHorizontal = TRUE, however, this is switched.
# If you don't want to include axis labels at all, just leave the double-quotes empty, i.e. "".
#=====[8]=====
xAxisLabel = ""
yAxisLabel = ""
#=============

# In the output, you will have a number of bars next to each other.
# The default is to have a small bit of space between neighboring bars that is 20% of the width of each of the individual bars
# Here, you can adjust this, inputting any percentage (running from 0 to 1 here rather than 0 to 100).
# If you want more space, you can increase this (e.g. up to 1, meaning that the space between bars is the same width as the bars themselves).
# More frequently, perhaps, you might want to turn this all the way down to 0 and have no spacing between bars at all.
#=====[9]=====
SpaceBetweenBars = 0.2
#=============

# If you would like your different bars to be filled with different colors, indicate here the name of the colors you would like to use.
# Type 'colors()' (without the single-quotes) at the R console and hit ENTER to see the list of all the color names R accepts.
# Be sure to surround the name of the colors with double-quotes, e.g. "blue".
# As before, we'll be using the c() function, so this will look something like c("red", "green", "blue").
# Make sure the number of colors matches the number of columns you have!
# If you leave this set to NULL, the bars in your barplot will all be filled with the same grey color.
#=====[10]=====
BarFillColors = NULL
#BarFillColors = c("red", "green", "blue", "orange")
#=============

# If you think the font size for either the number scale or the bar labels is too big or too small, you can adjust that here.
# These are expressed in terms of percents. Thus, for example, if one of these two kinds of labels is too big, you can set it to 0.9, or if one of these is too small, you can set it to 1.1
# Even if you do not want to adjust these, you must run these two lines to set them at 100% - the default.
#=====[11]=====
FontSizeAdjustment_NumberScale = 1
FontSizeAdjustment_BarLabels = 1
#=============

###############
# CREATE PLOT #
###############

# Finally, run the following line of code to apply all the parameters you just set and create your barplot.
#=====[12]=====
barplot(height=tapply(X=NumberData,INDEX=IdentifierLabels,FUN="mean"), width=1, space=SpaceBetweenBars, legend.text=NULL, names.arg=BarLabels, beside=TRUE, horiz=ShouldThisBarplotBeHorizontal, density=NULL, col=BarFillColors, border="black", main=MainTitle, sub=NULL, xlab=xAxisLabel, ylab=yAxisLabel, xlim=xAxisLimits, ylim=yAxisLimits, xpd=TRUE, log="", axes=TRUE, axisnames=TRUE, cex.axis=FontSizeAdjustment_NumberScale, cex.names=FontSizeAdjustment_BarLabels, plot=TRUE, axis.lty=0, offset=0, add=FALSE, args.legend = NULL)
#=============

# This step is optional. Run this extra line of code if you would like to add 'whiskers' of +/- 1 standard deviation to the bars, indicating the range of dispersion from the averages represented in the bars themselves.
# Since the original barplot was not created with these 'whiskers' in mind, they may extend outside the range of the plot and look strange. If so, just go back up to step 7 and adjust the x and y axis limits so the whiskers fit nicely into the plot.
#=====[15]=====
Means = tapply(X=NumberData,INDEX=IdentifierLabels,FUN="mean"); SDs = tapply(X=NumberData,INDEX=IdentifierLabels,FUN="sd"); Midpoints = barplot(height=Means, width=1, space=SpaceBetweenBars, beside=TRUE, plot=FALSE); MinusSD = Means-SDs; PlusSD = Means+SDs; if(ShouldThisBarplotBeHorizontal==FALSE){ x0 = Midpoints; x1 = Midpoints; y0 = MinusSD; y1 = PlusSD }; if(ShouldThisBarplotBeHorizontal==TRUE){ x0 = MinusSD; x1 = PlusSD; y0 = Midpoints; y1 = Midpoints }; arrows(x0=x0, y0=y0, x1=x1, y1=y1, length = 0.25, angle=90, code=3, col = "black", lty = "solid", lwd=1, xpd=TRUE)
#=============
