# Create barplot from one number column and two label columns
# by Aaron Albin
# Script began: 4/4/2012

###########################
# WHEN TO USE THIS SCRIPT #
########################### 

# Use this script when you have one single column of number data in your dataframe with two other columns containing the corresponding labels (two different sets of categories or conditions).
# You wish to represent the averages for all combinations of conditions as multiple bars (hence, in a 'barplot'), clustered and perhaps color-coded according to the two different sets of conditions.
# You also might want to add 'whiskers' to each of the bars to show the range of dispersion (the 'standard deviation') from this average.

# Note that this looks similar to, but is in fact different from, a histogram.

###############
# SELECT DATA #
###############

# Run this to have R tell you the names for each of the columns that it ended up with after importing the dataframe:
#=====[1]=====
colnames(Dataframe)
#=============

# Now, pick out which of these columns has your number data.
# Type the name of that column between the double-quotes below (leaving the other code untouched).
#=====[2]=====
NumberData = Dataframe[,"NumberColumn1"]
# NumberData = Dataframe[,"F4"]
#=============

# Here, pick out which two columns have your identifier labels.
# In the barplot you will be creating, the bars will be clustered together according to one of your two sets of conditions.
# Then, within each cluster, the bars will be color-coded according to your other set of conditions.
# Below, inside the double-quotes, type the names of the column names containing the identifier labels you would like to use for these two purposes.
#=====[3]=====
ClusteringConditions = Dataframe[,"LabelColumn1"]
ColorCodedConditions = Dataframe[,"LabelColumn2"]
# ClusteringConditions = Dataframe[,"CONSONAT"]; ColorCodedConditions = Dataframe[,"SYLLABIF"]
#=============

# Run all four of these lines of code to store the names you are using for your two different sets of identifier labels and also show them to you (so you can double-check that they are as you expect them to be).
#=====[4]=====
Labels_ClusteringConditions = levels(ClusteringConditions)
Labels_ColorCodedConditions = levels(ColorCodedConditions)
levels(ClusteringConditions)
levels(ColorCodedConditions)
#=============

# This step is optional - you can skip down to step 6 if you like without causing any problems.
# Below, for each of the labels you just saw in the console from the previous step, you can specify a new label for it using spaces or any special symbols you want. This might make for a more visually-appealing plot.
# If you want to include a line break in the middle of your label, this is written as '\n' (without the single-quotes). For example, "My Experimental\nCondition One".
# Below, we use the [c]ombine function 'c()'. This lets you make a list of as many column names as you want as long as they are separated by commas:
# For example, c("A","B") and c("A","B","C") and c("A","B","C","D") are all fine - you can include as many as you want.
# As you'll notice, for each column-name entry in this list, to make sure R knows we're dealing with names/letters and not numbers, we need to make sure and surround the column name with double-quotes.
# A few further notes:
# - It is important that you double-check that you have the same number of entries in this list as the one you just saw in the console from Step 4!
# - Also, keep the entries here in the same order as what you saw from step 4.
# - You can of course have a different number of entries for Labels_ClusteringConditions and Labels_ColorCodedConditions.
# - Make sure not to get mixed up about which are the Clustering Conditions and which are the Colo-Coded Conditions, though.
#=====[5]=====
Labels_ClusteringConditions = c("Cluster one", "Cluster two", "Cluster three")
Labels_ColorCodedConditions = c("Color-condition one", "Color-condition two")
# Labels_ClusteringConditions = c("The [b] sound", "The [p] sound"); Labels_ColorCodedConditions = c("Coda sound", "Onset sound")
#=============

###############################
# SPECIFY PARAMETERS FOR PLOT #
###############################

# Set the name for the main title (centered along the top of the plot) here.
#=====[6]=====
MainTitle = "Barplot"
#=============

# You can have R either create a vertical or horizontal barplot.
# This describes the direction that the bars run on the plot.
# If you make a vertical barplot, your bar labels will be on the bottom of the plot and the numerical scale will be on the left, with bars vertically sticking up. The first bar will be on the left.
# If you make a horizontal barplot, your bar labels will be on the left of the plot and the numerical scale will be on the bottom, with bars horizontally sticking out from the left. The first bar will be on the bottom, which may be opposite of what you'd expect.
# Here, indicate your decision in terms of a TRUE/FALSE decision of 'Should this barplot be horizontal?'
# Remember that TRUE or FALSE must be in all caps and must *not* be surrounded by quotes.
#=====[7]=====
ShouldThisBarplotBeHorizontal = FALSE
#=============

# Generally, it is a good idea to let R determine the limits (i.e. range) of the x (horizontal) axis and y (vertical) axis for you. 
# Leaving the values below as NULL will have this effect.
# However, if you would like to adjust them, you can do that here.
# Just say xAxisLimits or yAxisLimits equal to 'c( ___, ___ )', where the two '___'s indicate the lower and upper bound (respectively) of the range for the relevant axis. Make sure to put the lower bound first (unless you want the axis to be backwards/flip-flopped).
# Note that if ShouldThisBarplotBeHorizontal = FALSE, you will almost always want to keep xAxisLimits as NULL. Similarly, if ShouldThisBarplotBeHorizontal = TRUE, you will almost always want to keep yAxisLimits as NULL.
#=====[8]=====
xAxisLimits = NULL
yAxisLimits = NULL
#=============

# Below the double-quotes below, you can insert a few words you would like to use as the 'axis labels' for the x and y axes.
# If ShouldThisBarplotBeHorizontal = FALSE:
# - xAxisLabel is the superordinate term for whatever all the different bars/categories represent
# - yAxisLabel is the name of the thing being measured, which all your number-data represent
# If ShouldThisBarplotBeHorizontal = TRUE, however, this is switched.
# If you don't want to include axis labels at all, just leave the double-quotes empty, i.e. "".
#=====[9]=====
xAxisLabel = ""
yAxisLabel = ""
#=============

# In the output, you will have a number of bars next to each other, grouped into clusters according to your 'ClusteringConditions' specified above.
# The default is to have a small bit of space between clusters but no space between bars inside that same cluster.
# Here, you can adjust these two parameters.
# The units here are 'bar widths.
# Thus, for example, setting SpaceBetweenBars_WithinCluster to 0.5 would create spacing within clusters equal to half of one bar width.
# Similarly, setting SpaceBetweenBars_BetweenClusters to 1.5 would increase the spacing between clusters to 150% the width of a bar.
#=====[10]=====
SpaceBetweenBars_WithinCluster = 0
SpaceBetweenBars_BetweenClusters = 1
#=============

# Here is where you must indicate the names of the colors, corresponding to the 'ColorCodedConditions' you specified above, that you would like to use to fill in your bars.
# Type 'colors()' (without the single-quotes) at the R console and hit ENTER to see the list of all the color names R accepts.
# Be sure to surround the name of the colors with double-quotes, e.g. "blue".
# As before, we'll be using the c() function, so this will look something like c("red", "green", "blue").
# Make sure the number of colors matches the number of conditions you have! (If you forgot how many you have, go back up and run step 4 to check again.)
# If you leave this set to NULL, the bars in your barplot will be automatically assigned different shades of grey.
#=====[11]=====
BarFillColors = NULL
#BarFillColors = c("red", "blue")
#=============

# If you think the font size for either the number scale or the bar labels is too big or too small, you can adjust that here.
# These are expressed in terms of percents. Thus, for example, if one of these two kinds of labels is too big, you can set it to 0.9, or if one of these is too small, you can set it to 1.1
# Even if you do not want to adjust these, you must run these two lines to set them at 100% - the default.
#=====[12]=====
FontSizeAdjustment_NumberScale = 1
FontSizeAdjustment_BarLabels = 1
#=============

###############
# CREATE PLOT #
###############

# Finally, run the following line of code to apply all the parameters you just set and create your barplot.
#=====[13]=====
barplot(height=tapply(X=NumberData,INDEX=list(ColorCodedConditions,ClusteringConditions),FUN="mean"), width=1, space=c(SpaceBetweenBars_WithinCluster, SpaceBetweenBars_BetweenClusters), legend.text=NULL, names.arg=Labels_ClusteringConditions, beside=TRUE, horiz=ShouldThisBarplotBeHorizontal, density=NULL, col=BarFillColors, border="black", main=MainTitle, sub=NULL, xlab=xAxisLabel, ylab=yAxisLabel, xlim=xAxisLimits, ylim=yAxisLimits, xpd=TRUE, log="", axes=TRUE, axisnames=TRUE, cex.axis=FontSizeAdjustment_NumberScale, cex.names=FontSizeAdjustment_BarLabels, plot=TRUE, axis.lty=0, offset=0, add=FALSE, args.legend = NULL)
#=============

# This step is optional. Run this extra line of code if you would like to add 'whiskers' of +/- 1 standard deviation to the bars, indicating the range of dispersion from the averages represented in the bars themselves.
# Since the original barplot was not created with these 'whiskers' in mind, they may extend outside the range of the plot and look strange. If so, just go back up to step 8 and adjust the x and y axis limits so the whiskers fit nicely into the plot.
#=====[14]=====
Means = tapply(X=NumberData,INDEX=list(ColorCodedConditions,ClusteringConditions),FUN="mean"); SDs = tapply(X=NumberData,INDEX=list(ColorCodedConditions,ClusteringConditions),FUN="sd"); Midpoints = barplot(height=Means, width=1, space=c(SpaceBetweenBars_WithinCluster, SpaceBetweenBars_BetweenClusters), beside=TRUE, plot=FALSE); MinusSD = Means-SDs; PlusSD = Means+SDs; if(ShouldThisBarplotBeHorizontal==FALSE){ x0 = Midpoints; x1 = Midpoints; y0 = MinusSD; y1 = PlusSD }; if(ShouldThisBarplotBeHorizontal==TRUE){ x0 = MinusSD; x1 = PlusSD; y0 = Midpoints; y1 = Midpoints }; arrows(x0=c(x0), y0=c(y0), x1=c(x1), y1=c(y1), length = 0.25, angle=90, code=3, col = "black", lty = "solid", lwd=1, xpd=TRUE)
#=============

##############
# ADD LEGEND #
##############

# Proceed only if you would like to create another tiny window containing the legend for your barplot, indicating what the different bar colors mean.

# Indicate here the text you would like to have running across the top of your legend as its title.
# If you want to make a legend that has no title, simply put NULL here (*without* quotes!).
#=====[15]=====
LegendTitle = "Title"
#=============

# If you would like the different components of your legend (color+label pairs) stacked vertically, leave this FALSE.
# If you want them arranged horizontally, however, set this to TRUE (without quotes).
#=====[16]=====
HorizontalLegend = FALSE
#=============

# Now run this code to create a tiny pop-up window with your legend.
# Note: This will only work right if you define your own 'BarFillColors' in step 11.
#=====[17]=====
HorizontalSpan=(par("usr")[1]-par("usr")[2])/(par("plt")[1]-par("plt")[2]); LeftEdge=par("usr")[1]-HorizontalSpan*par("plt")[1]; RightEdge=LeftEdge+HorizontalSpan; VerticalSpan=(par("usr")[3]-par("usr")[4])/(par("plt")[3]-par("plt")[4]); BottomEdge=par("usr")[3]-VerticalSpan*par("plt")[3]; TopEdge=BottomEdge+VerticalSpan; Legend=legend(x=par("usr")[1],y=par("usr")[4],legend=Labels_ColorCodedConditions,fill=BarFillColors,title=LegendTitle,horiz=HorizontalLegend,plot=FALSE); width=par("fin")[1]*Legend$rect$w/(RightEdge-LeftEdge); height=par("fin")[2]*Legend$rect$h/(TopEdge-BottomEdge); OuterMargin=0.1; dev.new(width=width+2*OuterMargin,height=height+2*OuterMargin); par(mai=rep(OuterMargin,times=4)); plot.default(x=NULL,y=NULL,xlim=c(0,width),ylim=c(height,0),axes=FALSE,ann=FALSE,xaxs="i",yaxs="i"); legend(x=0,y=0,legend=Labels_ColorCodedConditions,fill=BarFillColors,border="black",bg="transparent",box.lty="solid",box.lwd=1,box.col="black",text.col="black",title=LegendTitle,title.col="black",horiz=HorizontalLegend,xpd=TRUE)
#=============
