# Create boxplot from multiple number columns
# by Aaron Albin
# Script began: 4/17/2012

###########################
# WHEN TO USE THIS SCRIPT #
########################### 

# Use this script when you have multiple columns of number data in your dataframe that you wish to plot as a boxplot.
# In this plot, you will get a visual representation of where the values in each column fall, containing the following components:
# [1] Points corresponding to outliers in your data, if you have any.
# [2] Lines shaped like the capital letter 'I' to indicate the threshold serving as the criteria for what an 'outlier' is.
# [3] A box indicating where the central 50% of the data points lie (once outliers are excluded)
#     (In more technical terms, this corresponds to the range between the 25% and 75% quartiles of the sample.)
# [4] A thick horizontal line indicating the median value of the data (again, with the outliers excluded)
# [5] Indentations into the box indicating how 'robust' this median is

###############
# SELECT DATA #
###############

# Run this to have R tell you the names for each of the columns that it ended up with after importing the dataframe:
#=====[1]=====
colnames(Dataframe)
#=============

# Specify the names of the columns that contain the number data you with to include in the boxplot.
# Below, we use the [c]ombine function 'c()'. This lets you make a list of as many column names as you want as long as they are separated by commas:
# For example, c("A","B") and c("A","B","C") and c("A","B","C","D") are all fine - you can include as many as you want.
# The order of your list will determine the order of the boxes in the resulting plot.
# As you'll notice, for each column-name entry in this list, to make sure R knows we're dealing with names/letters and not numbers, we need to make sure and surround the column name with double-quotes.
# Using this format, type up the names of the columns you wish to include in the boxplot below.
#=====[2]=====
NumberColumnNames=BoxLabels=c("NumberColumn1","NumberColumn2","NumberColumn3")
# NumberColumnNames=BoxLabels=c("F1","F2","F3","F4")
#=============

# This step is optional - you can skip down to step 4 if you like without causing any problems.
# As you'll recall, in the 'Import dataframe' script, it was explicitly recommended you not use spaces or any special symbols (besides '.' and '_') in your column names.
# Below, for each of the column names you specified in the previous step, you can specify a new label for it using spaces or any special symbols you want. This might make for a more visually-appealing plot.
# If you want to include a line break in the middle of your label, this is written as '\n' (without the single-quotes). For example, "My Experimental\nCondition One".
# Use the same rules here as above - separate entries in the list with commas, and surround each entry with double-quotes.
# NOTE: It is important that you double-check that you have the same number of entries in this list as the one you made for Step 2! Also, keep them in the same order.
#=====[3]=====
BoxLabels=c("My first column", "My second column", "My third column")
# BoxLabels=c("1st formant","2nd formant","3rd formant","4th formant")
#=============

# Now run the following code to extract out the columns you just selected
# Note: Having NA values in your data will not cause any problems, as the NAs will be simply ignored when creating the boxplot.
#=====[4]=====
NumberData=Dataframe[,NumberColumnNames]
#=============

###############################
# SPECIFY PARAMETERS FOR PLOT #
###############################

# Set the name for the main title (centered along the top of the plot) here.
#=====[5]=====
MainTitle = "Boxplot"
#=============

# You can have R either create a vertical or horizontal boxplot.
# This describes the overall orientation for the plot, i.e. in which direction the boxes will run.
# If you make a vertical boxplot, your labels will be on the bottom of the plot and the numerical scale will be on the left, with the boxes running up-and-down. The first box will be on the left.
# If you make a horizontal barplot, your labels will be on the left of the plot and the numerical scale will be on the bottom, with the boxes running left-and-right. The first bar will be on the bottom, which may be opposite of what you'd expect. If you want to re-order the columns to make them look better, you can do that above in steps 2 and 3.
# Here, indicate your decision in terms of a TRUE/FALSE decision of 'Should this boxplot be horizontal?'
# Remember that TRUE or FALSE must be in all caps and must *not* be surrounded by quotes.
#=====[6]=====
ShouldThisBoxplotBeHorizontal = FALSE
#=============

# Generally, it is a good idea to let R determine the limits (i.e. range) of the x (horizontal) axis and y (vertical) axis for you. 
# Leaving the values below as NULL will have this effect.
# However, if you would like to adjust them, you can do that here.
# Just say xAxisLimits or yAxisLimits equal to 'c( ___, ___ )', where the two '___'s indicate the lower and upper bound (respectively) of the range for the relevant axis. Make sure to put the lower bound first (unless you want the axis to be backwards/flip-flopped).
# Note that if ShouldThisBoxplotBeHorizontal = FALSE, you will almost always want to keep xAxisLimits as NULL. Similarly, if ShouldThisBoxplotBeHorizontal = TRUE, you will almost always want to keep yAxisLimits as NULL.
#=====[7]=====
xAxisLimits = NULL
yAxisLimits = NULL
#=============

# Below the double-quotes below, you can insert a few words you would like to use as the 'axis labels' for the x and y axes.
# If ShouldThisBoxplotBeHorizontal = FALSE:
# - xAxisLabel is the superordinate term for whatever all the different boxes/categories represent
# - yAxisLabel is the name of the thing being measured, which all your number-data represent
# If ShouldThisBoxplotBeHorizontal = TRUE, however, this is switched.
# If you don't want to include axis labels at all, just leave the double-quotes empty, i.e. "".
#=====[8]=====
xAxisLabel = ""
yAxisLabel = ""
#=============

# Here you can adjust the threshold for what constitutes an 'outlier'
# The default is 1.5, indicating 150% of the interquantile range (which visually corresponds to one-and-a-half times the length of the box).
# Once you produce your plot, if you see too many points treated as outliers, just increase this number (e.g. to 1.6 or more).
# Alternatively, if you feel that more data points ought to be treated as outliers than you are seeing, just increase this number (e.g. to 1.4 or less).
# It is also possible to *not* treat any data point as an outlier; to do this, just set OutlierThreshold to 0.
#=====[9]=====
OutlierThreshold=1.5
#=============

# You have the option of adding triangular indentations to your box that indicate how 'robust' the median in each box is given the data.
# The indentations represent plus or minus 1.58 times the interquartile range divided by the square root of the number of observations.
# This calculation was chosen because it effectively gives an approximation to the 95% confidence interval for any given median.
# This lets you visually check whether the indentations of two boxes overlap or not, which can be useful as tentative evidence that the two medians are substantially different.
#=====[10]=====
AddMedianRobustnessIndentations = FALSE
#=============

# Here you adjust the width of the boxes.
# If you set this to 1, neighboring boxes will be touching, which often is not visually appealing.
# Consequently, the default is set at 0.8, which typically looks good.
# However, if, up above in Step 2, you only read in 2 or 3 columns for analysis, you may want to reduce the width of your boxes (say, to 0.5) to make the plot look nice.
# The other components of each of the boxes in the plot (namely, the outlier threshold lines and the median robustness indentations) will automaticlly adjust accordingly.
#=====[11]=====
BoxWidth = 0.8
#=============

# If you would like your different boxes to be filled with different colors, indicate here the name of the colors you would like to use.
# Type 'colors()' (without the single-quotes) at the R console and hit ENTER to see the list of all the color names R accepts.
# Be sure to surround the name of the colors with double-quotes, e.g. "blue".
# As before, we'll be using the c() function, so this will look something like c("red", "green", "blue").
# Make sure the number of colors matches the number of columns you specified in step 2 above!
# If you leave this set to NULL, all the boxes will appear to be filled with just a plain white.
#=====[12]=====
BoxFillColors = NULL
#BoxFillColors = c("red", "green", "blue", "orange")
#=============

###############
# CREATE PLOT #
###############

# Finally, run the following line of code to apply all the parameters you just set and create your boxplot.
#=====[13]=====
boxplot(x=NumberData, names=BoxLabels, main=MainTitle, horizontal=ShouldThisBoxplotBeHorizontal, xlim=xAxisLimits, ylim=yAxisLimits, xlab=xAxisLabel, ylab=yAxisLabel, range=OutlierThreshold, notch=AddMedianRobustnessIndentations, boxwex=BoxWidth, boxfill=BoxFillColors, plot=TRUE, add=FALSE, show.names=TRUE, axes=TRUE, log="", frame.plot=TRUE, width=NULL, varwidth=FALSE, staplewex=0.5, notch.frac=0.5, border="black", medlty="solid", medlwd=3, medpch=NA, boxlty = "solid", boxlwd=1, whisklty="dashed", whisklwd=1, staplelty="solid", staplelwd=1, outline=TRUE, outpch=1, outcex=1, outlty="blank", pars=NULL)
#=============
