# Create X-Y scatterplot: Coded by color
# by Aaron Albin
# Script began: 4/11/2012

###########################
# WHEN TO USE THIS SCRIPT #
########################### 

# Use this script when both of the following is true:
# * Inside your dataframe, you have two sets of numeric data stored as two separate columns, and the individual data points inside the two are related to each other (or 'paired'). This is applicable, for instance, whenever a single event/phenomenon/subject simultaneously yields two data points.
# * You wish to plot these numbers onto a 'scatterplot', where the horizontal axis corresponds to one of your columns of numbers and the vertical axis corresponds to the other. Each individual pair of numbers, then, is represented as a point in this two-dimensional space.
# * You have one other additional column with labels in it (e.g. telling you which experimental condition each pair of data points came from), and you want to encode these labels into the plot as the different colors of the points in the scatterplot.

###############
# SELECT DATA #
###############

# Run this to have R tell you the names for each of the columns that it ended up with after importing the dataframe:
#=====[1]=====
colnames(Dataframe)
#=============

# Now, pick out which two of these columns have your number data.
# Type the names of those columns between the double-quotes below (leaving the other code untouched).
# Which you choose as 'NumberDataX' and which you choose as 'NumberDataY' is important here.
# The horizontal axis will correspond to the values you set as 'NumberDataX', and the vertical axis will correspond to the values you set as 'NumberDataY'.
# Throughout this script, it will be useful to keep this naming system in mind - x=horizontal and y=vertical.
#=====[2]=====
NumberDataX = Dataframe[,"NumberColumn1"]
NumberDataY = Dataframe[,"NumberColumn2"]
# NumberDataX = Dataframe[,"OPEN_DUR"]; NumberDataY = Dataframe[,"GLOTT_DUR"]
#=============

# Pick out which of your columns have your label data that you would like to encode as coloring of the points on the scatterplot.
# Type the names of that column between the double-quotes below (leaving the other code untouched).
#=====[3]=====
LabelColumn_PointColor = as.factor(Dataframe[,"LabelColumnName"])
# LabelColumn_PointColor = as.factor(Dataframe[,"CONSONAT"])
#=============

# Run these two lines to store the unique namess from the label column and also show them to you (so you can double-check that they are as you expect them to be).
#=====[4]=====
LegendLabels = levels(LabelColumn_PointColor)
levels(LabelColumn_PointColor)
#=============

# This step is optional - you can skip down to step 6 if you like without causing any problems.
# Below, for each of the column names you just saw in the console from the previous step, you can specify a new label for it using spaces or any special symbols you want. This might make for a more easy-to-read legend.
# If you want to include a line break in the middle of your label, this is written as '\n' (without the single-quotes). For example, "My Experimental\nCondition One".
# Below, we use the [c]ombine function 'c()'. This lets you make a list of as many column names as you want as long as they are separated by commas:
# For example, c("A","B") and c("A","B","C") and c("A","B","C","D") are all fine - you can include as many as you want.
# As you'll notice, for each column-name entry in this list, to make sure R knows we're dealing with names/letters and not numbers, we need to make sure and surround the column name with double-quotes.
# NOTE: It is important that you double-check that you have the same number of entries in this list as the one you just saw in the console from Step 4! Also, keep them in the same order.
#=====[5]=====
LegendLabels = c("My first condition", "My second condition", "My third condition")
# LegendLabels = c("The [b] sound", "The [p] sound")
#=============

###############################
# SPECIFY PARAMETERS FOR PLOT #
###############################

# Set the name for the main title (centered along the top of the plot) here.
#=====[6]=====
MainTitle = "Scatterplot"
#=============

# Generally, it is a good idea to let R determine the limits (i.e. range) of the x (horizontal) axis and y (vertical) axis for you. 
# Leaving the values below as NULL will have this effect.
# However, if you would like to adjust them, you can do that here.
# Just say xAxisLimits or yAxisLimits equal to 'c( ___, ___ )', where the two '___'s indicate the lower and upper bound (respectively) of the range for the relevant axis.
# If you want the axis to be backwards/flip-flopped, then but the upper bound first, followed by the lower bound, e.g. rather than c(5,100) say c(100,5).
#=====[7]=====
xAxisLimits = NULL
yAxisLimits = NULL
#=============

# Below the double-quotes below, you can insert a few words you would like to use as the 'axis labels' for the x and y axes.
# In the quotes after 'xAxisLabel', type the words(s) you want to see displayed beneath the x (horizontal) axis.
# In the quotes after 'yAxisLabel', type the word(s) you want to see displayed to the left of the y (vertical) axis.
# If you don't want to include an axis label at all for either of these, just leave the double-quotes empty, i.e. "", for that one.
#=====[8]=====
xAxisLabel = ""
yAxisLabel = ""
#=============

# In some applications, the interesting patterns in your numbers are happening at small numbers (e.g. under 10), but you have a few outliers (e.g. over 100) that are forcing R to zoom out far enough to encompass all the data, so far that you can't see the interesting patterns in your data in the small-number range.
# To overcome this problem, you can have R plot one or both of the axes 'logarithmically'.
# What this means is that, visually, smaller numbers will be 'given more space', whereas the higher numbers will be 'scrunched together' more.
# Below, indicate which axes
# PlotWhichAxesLogarithmically = "" means you don't want either axis to be plotted logarithmically
# PlotWhichAxesLogarithmically = "x" means you want the horizontal axis to be plotted logarithmically
# PlotWhichAxesLogarithmically = "y" means you want the vertical axis to be plotted logarithmically
# PlotWhichAxesLogarithmically = "xy" means you want *both* axes to be plotted logarithmically
# Note: If you choose to plot an axis logarithmically, any zeroes and any negative numbers will be omitted from the plot. (R will give you a warning telling you how many total points this was.) This is necessary because of how the underlying logarithmic transformation itself works.
#=====[9]=====
PlotWhichAxesLogarithmically = ""
#=============

# Choose here the symbol that you want to use for all the points that get plotted in the scatter plot. Each symbol is represented by a special 'code number':
#  1 = unfilled circle
#  0 = unfilled square
#  2 = unfilled triangle
#  5 = unfilled diamond
# 16 =   filled circle
# 15 =   filled square
# 17 =   filled triangle
# 18 =   filled diamond
#  3 = a plus sign (i.e. '+')
#  4 = an 'x' symbol
#  8 = a star composed of '+' and 'x' superimposed
# You should *not* surround any numbers like these with quotes.
# Indicate which one you like by typing the code number below after the equals sign (without quotes).
#=====[10]=====
PointSymbol = 1
#=============

# Run this line of code to see what labels are inside your label column selected earlier.
#=====[11]=====
levels(LabelColumn_PointColor)
#=============

# Here, you can choose the colors that you would like R to use when drawing all of your points on the scatterplot.
# Be sure to surround the name of the colors with double-quotes, e.g. "black".
# Type 'colors()' (without the single-quotes) at the R console and hit ENTER to see the list of all the color names R accepts.
# Hint: Colors like red, green, blue, cyan, magenta, yellow, and many others can be easily darkened by putting a number after them.
#       Taking green for instance, "green" is the same thing as "green1".
#       To darken the color, you can make it "green2", "green3" or "green4".
#       This stops at 4, however - there is no "green5" or anything beyond that.
#       Grey colors are different. There is "grey0" (black) to "grey100" (white) and everything in-between.
#          This gives you lots of flexibility in specifying greyscale colors. (Note: The spelling 'gray' also works.)
# Below, inside the 'combine' function 'c()', list which color you would like associated with each character by giving the color name for each in quotes.
# The ordering will match the order of labels you just saw in the last step - in other words, the first thing there will be the first color here, the second will be the second, and so on.
# It's perfectly OK to have more colors specified here than you will actually use. Think of this list here as the full default hierarchy that is referred to in making each plot.
#=====[12]=====
#             [1]     [2]   [3]      [4]    [5]    [6]       [7]      [8]    
PointColors=c("black","red","green3","blue","cyan","magenta","yellow","gray")
#=============

# If you would like your points to be bigger or smaller, the following lets you adjust their size.
# '1' represents 100% of the default size, so if you want to make the points larger you could type, for example, 1.5 (i.e. 150%). Similarly, if you want to make them smaller you could type 0.5 (i.e. 50%).
#=====[13]=====
PointSizeModifier = 1
#=============

###############################
# SPECIFY PARAMETERS FOR GRID #
###############################

# This section lets you add a 'grid' to the plot, running across the main plotting region to help you keep track of where things align with the axes.
# R will draw lines wherever the 'tick marks' are on both axes. If you wish to adjust these, change 'xAxisLimits' and 'yAxisLimits' above.

# Choose the type of line you would like to use for your gridlines. You have the following six choices:
#  "solid"    = -----------------------------------------------
#  "dashed"   = ----    ----    ----    ----    ----    ----   
#  "dotted"   = -   -   -   -   -   -   -   -   -   -   -   -  
#  "dotdash"  = -   ----   -   ----   -   ----   -   ----   -  
#  "longdash" = -------   -------   -------   -------   -------
#  "twodash"  = --  ------  --  ------  --  ------  --  ------ 
# If you do not want gridlines on your plot, choose "blank" instead.
#=====[14]=====
GridlineType = "dotted"
#=============

# Choose the color for your grid lines. See step 12 for help.
#=====[15]=====
GridlineColor = "lightgrey"
#=============

# If you would like your gridlines to be thicker, the following lets you increase its width.
# '1' represents 100% of the default thickness, so for example, you could type 2 to make it 200% that thickness.
#=====[16]=====
GridlineWidthModifier = 1
#=============

###############
# CREATE PLOT #
###############

# Finally, run the following line of code to apply all the parameters you just set and create your scatterplot.
#=====[17]=====
plot.default(x=NumberDataX, y=NumberDataY, main=MainTitle, xlim=xAxisLimits, ylim=yAxisLimits, xlab=xAxisLabel, ylab=yAxisLabel, log=PlotWhichAxesLogarithmically, pch=PointSymbol, col=PointColors[LabelColumn_PointColor], cex=PointSizeModifier, type="p", ann=TRUE, axes=TRUE, frame.plot=TRUE, panel.first=grid(lty=GridlineType, col=GridlineColor, lwd=GridlineWidthModifier), panel.last=NULL, asp=NA)
#=============

#######################
# FIT REGRESSION LINE #
#######################

# This section is for when you want to add a 'best-fit' regression lines to the plot for each color-grouping of points.
# The colors of the regression lines will correspond to the colors of the points you specified up above

# Choose the types of line you would like to draw for each of the regression lines. See step 14 for help.
#=====[18]=====
RegressionLineType = c("solid","dashed","dotted","dotdash","longdash","twodash")
#=============

# If you would like your line to be thicker, the following lets you increase its width.
# '1' represents 100% of the default thickness, so for example, you could type 2 to make it 200% that thickness.
#=====[19]=====
RegressionLineWidthModifier = 1
#=============

# The following code actually adds the lines to the plot.
#=====[20]=====
Groups = levels(LabelColumn_PointColor); nGroups = length(Groups); for(EachGroup in 1:nGroups){Filter = ( LabelColumn_PointColor == Groups[EachGroup] ); Regression = lm(formula=NumberDataY[Filter]~NumberDataX[Filter]); abline(reg=Regression,lty=RegressionLineType[EachGroup],col=PointColors[EachGroup],lwd=RegressionLineWidthModifier)}
#=============


##############
# ADD LEGEND #
##############

# Proceed only if you would like to create another tiny window containing the legend for your scatterplot, indicating what the different colors mean.

# Indicate here the text you would like to have running across the top of your legend as its title.
# If you want to make a legend that has no title, simply put NULL here (*without* quotes!).
#=====[16]=====
LegendTitle = "Title"
#=============

# If you would like the different components of your legend (color+label pairs) stacked vertically, leave this FALSE.
# If you want them arranged horizontally, however, set this to TRUE (without quotes).
#=====[17]=====
HorizontalLegend = FALSE
#=============

# Now run this code to create a tiny pop-up window with your legend.
#=====[18]=====
if(as.logical(sum(objects()=="RegressionLineType"))){LegendLines=RegressionLineType[1:length(levels(LabelColumn_PointColor))]}else{LegendLines=NULL}; HorizontalSpan=(par("usr")[1]-par("usr")[2])/(par("plt")[1]-par("plt")[2]); LeftEdge=par("usr")[1]-HorizontalSpan*par("plt")[1]; RightEdge=LeftEdge+HorizontalSpan; VerticalSpan=(par("usr")[3]-par("usr")[4])/(par("plt")[3]-par("plt")[4]); BottomEdge=par("usr")[3]-VerticalSpan*par("plt")[3]; TopEdge=BottomEdge+VerticalSpan; Legend=legend(x=par("usr")[1],y=par("usr")[4],legend=LegendLabels,pch=PointSymbol,lty=LegendLines,pt.cex=PointSizeModifier,title=LegendTitle,horiz=HorizontalLegend,plot=FALSE); width=par("fin")[1]*Legend$rect$w/(RightEdge-LeftEdge); height=par("fin")[2]*Legend$rect$h/(TopEdge-BottomEdge); OuterMargin=0.1; dev.new(width=width+2*OuterMargin,height=height+2*OuterMargin); par(mai=rep(OuterMargin,times=4)); plot.default(x=NULL,y=NULL,xlim=c(0,width),ylim=c(height,0),axes=FALSE,ann=FALSE,xaxs="i",yaxs="i"); legend(x=0,y=0,legend=LegendLabels,pch=PointSymbol,col=PointColors[1:length(levels(LabelColumn_PointColor))],lty=LegendLines,pt.cex=PointSizeModifier,title=LegendTitle,horiz=HorizontalLegend,border="black",bg="transparent",box.lty="solid",box.lwd=1,box.col="black",text.col="black",title.col="black",xpd=TRUE)
#=============
