# Import dataframe
# by Aaron Albin

###########################
# WHEN TO USE THIS SCRIPT #
########################### 

# This script is the first step in any analysis. It is assumed you have already taken the observations/measurements from your experiment and organized them in a spreadsheet program like Microsoft Excel. This script takes that matrix of numbers and labels and imports it into R as something known as a 'data frame'. While there are ways for R to interface directly with Excel, the best approach is to create text files with your data. One way of doing this is simply copying and pasting the range of cells into a text editor like Notepad - the divisions between cells will be retained in the form of tabs. This is often the simplest and fastest way.
# Another way of getting data into a textfile this is to have Excel create one for you. To do this, you first need to either create a Excel file with only one sheet, or else make the sheet with your data the 'active sheet' (i.e. it's clicked-on and visible). Then, go to the 'File' menu in Excel, click 'Save as...', and choose either 'Text (tab-delimited)' or CSV (comma-delimited).

#########################
# SET WORKING DIRECTORY #
######################### 

# This script requires you to have all data files you'll be feeding into the analysis inside one single folder on your computer, which we'll call the 'Working Directory'.
# Spell out the full path to this folder on your computer here, surrounding the whole thing with double-quotes.
# Make sure to use forward slashes (/) and NOT backward slashes (\).
# (The path can optionally end with a forward slash (/), but this isn't necessary.)
# The 'setwd()' function [SET]s the [W]orking [D]irectory to the folder you specify.
#=====[1]=====
setwd("C:/Users/...")
#=============

###################
# IMPORT THE DATA #
################### 

# This script requires there to be a textfile in the working directory with the dataframe containing the numbers you wish to run through the analysis.
# Run this line of code to have R tell you all the files that are in the folder you just specified.
# If you see the file you want to import into R, then move on to the next step.
#=====[2]=====
list.files()
#=============

# Here, type out the name of the textfile you want to import.
#=====[3]=====
DataSourceFilename = "X-lang_Regression.txt"
#=============

# Inside this textfile, does the first line contain the names of each of the columns?
# If so, put TRUE here, and if not, put FALSE. (It's crucial that these have no quotes around them and are written in all caps.)
#=====[4]=====
FirstLineIsColumnNames = TRUE
#=============

# Inside this textfile, each separate row of the dataframe must be appear on a different line inside the text file.
# Within each of these lines, there must be a single uniform way that consecutive values (i.e. the different 'cells' that will eventually be in different columns) are separated from each other. Here are some examples:
# - Tab-separated:       1.23	4.56	7.89	...
# - Space-separated:     1.23 4.56 7.89 ...
# - Comma-separated:     1.23,4.56,7.89,...
# - Semicolon-separated: 1.23;4.56;7.89;...
# Indicate here which of these you are using. Use "\t" for 'Tab-separated', " " for 'Space-separated', "," for 'Comma-separated', or ";" for 'Semicolon-separated'.
# Two things to double-check:
# * Make sure that you don't have multiple separators next to each other (e.g. two or three spaces between neighboring values).
# * Make sure that all decimals are marked with periods (e.g. 1.23), not with commas (e.g. 1,23).
# * If you have any empty cells (i.e. with a missing data point), then fill up those cells with the word NA (no slash - i.e. NOT 'N/A').
# Note: If you set this to "", you can have R guess which Separator you are using.
#=====[5]=====
Separator = "\t"
#=============

# In dataframes, you can have both text (i.e. data that is not numeric in nature but is rather 'character strings' made up of letters, etc.) as well as numbers, but these two different kinds of data need to be stored as separate columns.
# If you do have any columns that contain text, you can optionally surround each text string with single-quotes (') or double-quotes (") in your input textfile to make sure R parses the file right. (Don't mix the two, though!) This is by no means required, though - since you specified the separator above, R will typically be able to figure this out on its own.
# If you do use quotes, you can include essentially whatever you like inside the quotes, even the Separator that you chose.
# Here's an example, with column names included:

# FirstColumnName    | SecondColumnName | ...
# -------------------------------------------
# "Text label one"   | 123.456          | ...
# "Text label two"   | 234.567          | ...
# "Text label three" | 345.678          | ...

# If this were tab-delimited, it would look like this:
# FirstColumnName	SecondColumnName	...
# "Text label one"	123.456	...
# "Text label two"	234.567	...
# "Text label three"	345.678	...

# If you have any holes in the data, don't just leave that cell blank. Instead, its safer to explicitly mark that cell as 'NA' in the raw data.
# If you have any blank lines, they will be skipped.
# If you have any rows with a different number of cells in them than all the others (i.e. there is an extra value stuck in there, or you forgot to mark a missing cell with an NA), then R will fill in the extra spaces with NAs.

# The 'read.table()' function reads in a 'table' from a textfile, which we will store in a 'dataframe' object within R. We will make two copies of this, one called 'Dataframe' plus another called 'RawDataframe'. (The latter will be used later in the script if you need to select a specific subset of rows - see below.)

#=====[6]=====
Dataframe = RawDataframe = read.table( file=DataSourceFilename, header=FirstLineIsColumnNames, sep=Separator, quote="\"", dec=".", row.names=NULL, as.is=FALSE, na.strings="NA", colClasses=NA, nrows=0, skip=0, check.names=FALSE, fill=TRUE, strip.white=TRUE, blank.lines.skip=TRUE, comment.char="", allowEscapes=FALSE, flush=FALSE, stringsAsFactors=TRUE, fileEncoding="", encoding="unknown")
#=============

# Run this to have a look at what you just imported to see if everything looks all right, e.g. if everything is off by one column.
#=====[7]=====
fix(Dataframe)
#=============
# Pay special attention if there are any NAs in your data that should not be there (especially on the right side of the dataframe).
# If something is wrong, go back into your source data files (in Microsoft Excel, etc.) and make the necessary fixes before proceeding.
# (This fix() function technically lets you make the fixes here directly in R, but it's a good idea to go back and fix the problem at its source.)
# Otherwise, now that you are aware of what your data structure looks like, you can now enter the next stage of your analysis.

#######################################
# SELECTING A SPECIFIC SUBSET OF ROWS #
#######################################

# This section of the script is for when you want to look at only certain rows inside the data you read in (rather than entire columns).
# (If this is not the case, you can skip the rest of this script and go directly to the other scripts for the specific analyses you want to conduct.)

# To get started, run this code to have R tell you the names for each of the columns that it ended up with after importing the data frame:
#=====[8]=====
colnames(RawDataframe)
#=============

# Now you'll need to tell R which rows should be retained. Here are the different comparison 'operators' that you can use:
# == 'is equal to' (Can be used for both numbers and text)
# != 'is not equal to' (Can be used for both numbers and text)
# >  'is greater than'
# <  'is less than'
# >= 'is greater than or equal to'
# <= 'is less than or equal to'

# And here are some examples:
# Dataframe[,"Condition"] == "Condition A"
# Dataframe[,"Group"] != "Group B"
# Dataframe[,"Measurement"] >  12.34
# Dataframe[,"Index"] <= 5

# You can add multiple filter conditions at once with & ('and') or | ('or'). For example:
#    Dataframe[,"Condition"] == "Condition A" & Dataframe[,"Measurement"] >  12.34
#     - Here, with 'and', BOTH of the two conditions have to be satisfied in order for the row to be retained.
#    Dataframe[,"Group"] != "Group B" | Dataframe[,"Index"] <= 5
#     - Here, with 'or', the row will be retained whenever EITHER (or both) of the two conditions is satisfied.

#=====[9]=====
RowFilter = RawDataframe[,"LabelColumn1"] == "Value"
# RowFilter = RawDataframe[,"NumberColumn1"] > 2
#=============

# Now run this line to extract out just the rows you have selected.
#=====[10]=====
Dataframe = RawDataframe[RowFilter,]
#=============

# Now go back up to step 7 and run 'fix(Dataframe)' again to see if you've obtained exactly what you were hoping for.
# If everything looks all right, then go ahead and move on to the scripts for the specific analyses you are interested in.

#______________________________
# APPENDIX - FOR ADVANCED USERS

# Below you will find a description of all the settings used above in the call to the read.table() function.

# file=DataSourceFilename,       # Filename (You specify this in the script above.)
# header=FirstLineIsColumnNames, # Whether there is a header or not (You specify this in the script above.)
# sep=Separator,                 # What the 'separator character' is (You specify this in the script above.)
# quote="\"",                    # What is allowed as a 'quoting character', which can be used to mark where non-number text begins and ends
# dec="."                        # What the decimal character is (e.g. a period, comma, etc.)
# row.names=NULL                 # Whether the first column of your data frame has row names
# as.is=TRUE                     # Whether R should store a text column as a 'factor' (as.is=TRUE) or as a 'character vector' (as.is=FALSE)
# na.strings="NA"                # How you indicate holes in your data (e,g, with the letters 'NA')
# colClasses=NA                  # What kind of data is in each of your columns (text, number, etc.); setting this to NA tells R to figure this out automatically
# nrows=0                        # How many rows you want to read in; setting this to an invalid number (like 0) tells R to read in all rows.
# skip=0                         # How many rows from the top of the textfile you want to skip in the reading-in process; setting this to an invalid number (like 0) tells R not to skip anything and, rather, to read in all the information from the very first line of the textfile
# check.names=FALSE              # Whether R should 'screen' your column names and re-format them so they conform to R's rules for variable names (check.names=TRUE) or not (check.names=FALSE)
# fill=TRUE                      # Whether R should add NAs to the end of a row if one row has fewer items in it than there are columns
# strip.white=TRUE               # Whether R should 'strip' (i.e. eliminate) extra 'white space' (e.g. spaces) that you have at the beginning or end of a text string
# blank.lines.skip=TRUE          # Whether you want R to ignore blank lines in the input textfile
# comment.char=""                # What comment character you used in the input text file (e.g. '#'); Setting this to "" tells R that you have no comments and rather everything should be read verbatim
# allowEscapes=FALSE             # If you have things like '\n' in your textfile, whether these should be interpreted as plain text (allowEscapes=FALSE) or as their C-style special meanings (e.g. 'new line' for '\n')
# flush=FALSE                    # Whether you include a comment at the bottom of the input text file and would like R to 'flush' that line to the console when reading it in.
# stringsAsFactors=FALSE         # Nearly the same as 'as.is' above
# fileEncoding=""                # "Declares the encoding used on a file so the character data can be re-encoded"; Setting this to "" tells R not to re-encode
# encoding="unknown"             # What encoding should be assumed for character strings in the input (so that R can handle encoded strings in their native encoding); Setting this to "unknown" tells R to figure this out automatically
