# Unpaired two sample/group/distribution t-test from one number column and one label column
# by Aaron Albin
# Script began: 2/1/2012

###########################
# WHEN TO USE THIS SCRIPT #
########################### 

# Use this script if the following is true:

# A) Inside your dataframe, you have two sets of numeric data stored together in one column, with an identifier label (e.g. "Condition A" vs. "Condition B") signaling which number goes in which group in a second column. (Make sure your identifier labels don't contain any special characters (just letters, numbers, periods, and underscores).)

# B) You only want to analyze a subset of the data, e.g. excluding all cases of a "Condition C" in order to contrast specifically "Condition A" vs. "Condition B".
# C) If you were to repeat your experiment over and over again an infinite number of times (with the intention of collecting the exact same number of data points as you currently have each time) and then calculate the average of your two sets of data for every one of those hypothetical datasets, you would be interested in knowing:
# * What the difference in averages would end up being most of the time (This is the 'confidence interval'.)
# * The probability that the difference in averages, or any even bigger difference, would be the result of nothing more than chance, i.e. random noise. (This is the 'p-value'.)

# Several examples:
# - Do men and women do differently on a test of spatial ability? 
# - Does one drug treat headache pain more effectively than another? 
# - Do College Juniors sleep more than College Seniors? 
# Source: http://www.wadsworth.com/psychology_d/special_features/ext/workshops/t_testbetween1.html

###############
# SELECT DATA #
###############

# Run this to have R tell you the names for each of the columns that it ended up with:
#=====[1]=====
colnames(Dataframe)
#=============

# Now, pick out which of these columns has your number data for the t-test and which has the identifier labels. Type the name of those columns between the double-quotes below.
#=====[2]=====
NumberColumnName  = "NumberColumn1"
LabelColumnName = "LabelColumn"
# NumberColumnName  = "VOT1"; LabelColumnName = "CONSONAT"

#=============

# Now run this to extract out the specific columns data you selected
#=====[3]=====
NumberData = Dataframe[,NumberColumnName]
IdentifierLabels = Dataframe[,LabelColumnName]
#=============

# By this point, there must be a two-way contrast in the identifier labels or else the t-test will not work.
# In order to make sure you are working with just the two you will need, run the code below.
# In it, the 'unique()' function shows you a list of all the unique entries you have inside the 'IdentifierLabels' object.
#=====[4]=====
unique(IdentifierLabels)
#=============

# Below, specify the two things you want to compare from the list of labels just produced.
# If you already have only two labels, then just specify them here anyways.
# Make sure you don't get capital and lower-case letters mixed up.
#=====[5]=====
LabelA = "Value1"
LabelB = "Value2"
#LabelA="b";LabelB="p"
#=============

# Now finally run the code below to use your selected two values to 'subset' only the specific rows you will need for this particular comparison from your larger dataframe.
#=====[6]=====
Subset = IdentifierLabels == LabelA |  IdentifierLabels == LabelB
#=============

###########################
# SET UP AND RUN ANALYSIS #
###########################

# Here, specify the 'true value' of the difference in averages between your 'X' and 'Y' datasets (X minus Y) under the null hypothesis. This will normally be 0.
#=====[7]=====
DifferenceInAverages_NullHypothesis = 0
#=============

# Here, specify the 'directionality' of the alternative hypothesis. You have three choices:
# * "two.sided" (Note the period!) - Your alternative hypothesis is that the difference in averages is NOT EQUAL TO the number specified immediately above for the null hypothesis. This is also called a 'two-tailed' t-test.
# * "greater" - Your alternative hypothesis is that the difference in averages is GREATER THAN the number specified immediately above for the null hypothesis. This is one kind of 'one-tailed' t-test.
# * "less" - Your alternative hypothesis is that the difference in averages is LESS THAN the number specified immediately above for the null hypothesis. This is one kind of 'one-tailed' t-test.
#=====[8]=====
Directionality_AlternativeHypothesis = "two.sided"
#=============

# In the t-test, you can choose whether you want to assume that variances of the two samples are equal. To determine whether this assumption is valid or not, the following calculates the variance of each of the two distributions for you.
#=====[9]=====
var(NumberData[IdentifierLabels == LabelA])
var(NumberData[IdentifierLabels == LabelB])
#=============

# Here, indicate whether you want to make the assumption that the variances are equal.
# If the two numbers calculated above are reasonably close to each other, then choose TRUE, and the variances from the two distribution will be pooled together.
# If the two numbers calculated above are not very close to each other, then choose FALSE, and the variance will be estimated separately for both groups. Moreover, the Welch-Satterthwaite approximation to the degrees of freedom will be used.
#=====[10]=====
VariancesEqual = FALSE
#=============

# In the output, it gives you a confidence interval. R defaults to a 95% confidence interval, as this is common practice in the literature, but you can adjust the percentage here if you like:
#=====[11]=====
PercentageForConfidenceInterval = 0.95
#=============

# Now run the actual 't.test()' function. 
# Note that R will issue an error message if the input data is unvarying/constant.
#=====[12]=====
Results = t.test( formula=NumberData~IdentifierLabels, subset=Subset, na.action = na.fail,alternative=Directionality_AlternativeHypothesis, mu=DifferenceInAverages_NullHypothesis, paired=FALSE, var.equal=VariancesEqual, conf.level=PercentageForConfidenceInterval)
#=============

#######################
# INSPECT THE RESULTS #
#######################

# Run the following code for a reformatted, easy-to-read representation of the results:
#=====[13]=====
{if(VariancesEqual==TRUE){VarianceAssumption="The analysis assumed that the two distributions each had the same variance.\n"}else{VarianceAssumption="The analysis assumed the two distributions have different variances.\n"}
if(as.character(Results$alternative)=="two.sided"){AlternativeText=paste("The true difference in means is not equal to ", as.numeric(Results$null.value),".",sep="")}
if(as.character(Results$alternative)=="greater"){AlternativeText=paste("The true difference in means is greater than ", as.numeric(Results$null.value),".",sep="")}
if(as.character(Results$alternative)=="less"){AlternativeText=paste("The true difference in means is less than ", as.numeric(Results$null.value),".",sep="")}
cat("\n\nDESCRIPTIVE STATISTICS\n~~~~~~~~~~~~~~~~~~~~~~\nThe mean of " ,LabelA," is ", as.numeric(Results$estimate)[1],".\nThe mean of ",LabelB," is ",as.numeric(Results$estimate)[2],".\nHence the actual calculated difference in means\n   (",LabelA," minus ", LabelB,") is ", as.numeric(Results$estimate[1] - Results$estimate[2]),".\n\nTEST THAT WAS RUN\n~~~~~~~~~~~~~~~~~\nAn Unpaired ", Results$method, " was performed on ", LabelA," and ", LabelB,".\n", " - Null hypothesis: The true difference in means is ", as.numeric(Results$null.value),".\n - Alternative hypothesis: ",AlternativeText,"\n",VarianceAssumption,"\nRESULTS\n~~~~~~~\nThe ",100*PercentageForConfidenceInterval,"% confidence interval for the difference in means ranges\n   from ", as.numeric(Results$conf.int[1]), " to ", as.numeric(Results$conf.int[2]),".\nt(", as.numeric(Results$parameter), ")=", as.numeric(Results$statistic), ", p=",as.numeric(Results$p.value),"\n\n\n",sep="")}
#=============

# Here is R's default display of the results:
#=====[14]=====
show(Results)
#=============
