# Paired two-sample/group/distribution t-test from two number columns
# by Aaron Albin
# Script began: 2/14/2012

###########################
# WHEN TO USE THIS SCRIPT #
########################### 

# Use this script if you can answer 'yes' to all of the following questions:

# A) Inside your dataframe, you have two sets of numeric data stored as two separate columns, and  the individual data points inside the two are related to each other (or 'paired'). This is applicable, for instance, whenever a single event/phenomenon/subject simultaneously yields two data points.
# A) If you were to repeat your experiment over and over again an infinite number of times (with the intention of collecting the exact same number of data points as you currently have each time) and then calculate the average of your two sets of data for every one of those hypothetical datasets, you would be interested in knowing:
# * What the difference in averages would end up being most of the time (This is the 'confidence interval'.)
# * The probability that the difference in averages, or any even bigger difference, would be the result of nothing more than chance, i.e. random noise. (This is the 'p-value'.)

# Example:
# * You run ten subjects through an experiment and tabulate their performance score into one number, then run the same ten subjects one week later and run the same analysis. In this way, you would have 'paired' data points, as each subject provided two performance scores that are intrinsically 'tied together'.

###############
# SELECT DATA #
###############

# Run this to have R tell you the names for each of the columns that it ended up with:
#=====[1]=====
colnames(Dataframe)
#=============

# Now, pick out which two columns have your number data for the t-test and type the names of those columns between the double-quotes below.
# It doesn't matter which you choose to be A and which you choose to be B  (as long as you remember which is which if you specify something other than 0 for Step 6).
#=====[2]=====
ColumnNameA = "NumberColumn1"
ColumnNameB = "NumberColumn2"
# ColumnNameA = "J_ANS_B"; ColumnNameB = "K_ANS_B"
#=============

# Now run this to extract out the specific columns data you selected
#=====[3]=====
NumberDataA = Dataframe[,ColumnNameA]
NumberDataB = Dataframe[,ColumnNameB]
#=============

###########################
# SET UP AND RUN ANALYSIS #
###########################

# Here, specify the 'true value' of the difference in averages between your 'NumberDataA' and 'NumberDataB' datasets (A minus B) under the null hypothesis. This will normally be 0.
#=====[4]=====
DifferenceInAverages_NullHypothesis = 0
#=============

# Here, specify the 'directionality' of the alternative hypothesis. You have three choices:
# * "two.sided" (Note the period!) - Your alternative hypothesis is that the difference in averages is NOT EQUAL TO the number specified immediately above for the null hypothesis. This is also called a 'two-tailed' t-test.
# * "greater" - Your alternative hypothesis is that the difference in averages is GREATER THAN the number specified immediately above for the null hypothesis. This is one kind of 'one-tailed' t-test.
# * "less" - Your alternative hypothesis is that the difference in averages is LESS THAN the number specified immediately above for the null hypothesis. This is one kind of 'one-tailed' t-test.
#=====[5]=====
Directionality_AlternativeHypothesis = "less"
#=============

# In the output, it gives you a confidence interval. R defaults to a 95% confidence interval, as this is common practice in the literature, but you can adjust the percentage here if you like:
#=====[6]=====
PercentageForConfidenceInterval = 0.95
#=============

# Now run the actual 't.test()' function. 
# Note that R will issue an error message if the input data is unvarying/constant.
#=====[7]=====
Results = t.test( x=NumberDataA, y=NumberDataB, alternative=Directionality_AlternativeHypothesis, mu=DifferenceInAverages_NullHypothesis, paired=TRUE, var.equal=VariancesEqual, conf.level=PercentageForConfidenceInterval)
#=============

#######################
# INSPECT THE RESULTS #
#######################

# Run the following code for a reformatted, easy-to-read representation of the results:
#=====[8]=====
{if(as.character(Results$alternative)=="two.sided"){AlternativeText=paste("The true average difference is not equal to ", as.numeric(Results$null.value),".",sep="")}
if(as.character(Results$alternative)=="greater"){AlternativeText=paste("The true average difference is greater than ", as.numeric(Results$null.value),".",sep="")}
if(as.character(Results$alternative)=="less"){AlternativeText=paste("The true average difference is less than ", as.numeric(Results$null.value),".",sep="")}
cat("\n\nDESCRIPTIVE STATISTICS\n~~~~~~~~~~~~~~~~~~~~~~\nAcross all the paired data points, the average difference\n   (",ColumnNameA," minus ", ColumnNameB,") is ", as.numeric(Results$estimate),".\n\nTEST THAT WAS RUN\n~~~~~~~~~~~~~~~~~\nA ", Results$method, " was performed on ", ColumnNameA," and ", ColumnNameB,".\n", " - Null hypothesis: The true average difference is ", as.numeric(Results$null.value),".\n - Alternative hypothesis: ",AlternativeText,"\n\nRESULTS\n~~~~~~~\nThe ",100*PercentageForConfidenceInterval,"% confidence interval for the average difference ranges\n   from ", as.numeric(Results$conf.int[1]), " to ", as.numeric(Results$conf.int[2]),".\nt(", as.numeric(Results$parameter), ")=", as.numeric(Results$statistic), ", p=",as.numeric(Results$p.value),"\n\n\n",sep="")}
#=============

# Here is R's default display of the results:
#=====[9]=====
show(Results)
#=============
