# Unpaired two sample/group/distribution t-test from two number columns
# by Aaron Albin
# Script began: 1/18/2012

###########################
# WHEN TO USE THIS SCRIPT #
########################### 

# Use this script if you can answer 'yes' to all of the following questions:

# A) Inside your dataframe, you have two sets of numeric data stored as two separate columns, and the individual data points inside the two are completely unrelated to each other.
# A) If you were to repeat your experiment over and over again an infinite number of times (with the intention of collecting the exact same number of data points as you currently have each time) and then calculate the average of your two sets of data for every one of those hypothetical datasets, you would be interested in knowing:
# * What the difference in averages would end up being most of the time (This is the 'confidence interval'.)
# * The probability that the difference in averages, or any even bigger difference, would be the result of nothing more than chance, i.e. random noise. (This is the 'p-value'.)

# Several examples:
# - Do men and women do differently on a test of spatial ability? 
# - Does one drug treat headache pain more effectively than another? 
# - Do College Juniors sleep more than College Seniors? 
# Source: http://www.wadsworth.com/psychology_d/special_features/ext/workshops/t_testbetween1.html

###############
# SELECT DATA #
###############

# Run this to have R tell you the names for each of the columns that it ended up with:
#=====[1]=====
colnames(Dataframe)
#=============

# Now, pick out which two columns have your number data for the t-test and type the names of those columns between the double-quotes below.
# It doesn't matter which you choose to be A and which you choose to be B (as long as you remember which is which if you specify something other than 0 for Step 4).
#=====[2]=====
ColumnNameA = "NumberColumn1"
ColumnNameB = "NumberColumn2"
# ColumnNameA="F1"; ColumnNameB="F2"
#=============

# Run this to extract out the specific columns data you selected
#=====[3]=====
NumberDataA = Dataframe[,ColumnNameA]
NumberDataB = Dataframe[,ColumnNameB]
#=============

###########################
# SET UP AND RUN ANALYSIS #
###########################

# Here, specify the 'true value' of the difference in averages between your 'NumberDataA' and 'NumberDataB' datasets (A minus B) under the null hypothesis. This will normally be 0.
#=====[4]=====
DifferenceInAverages_NullHypothesis = 0
#=============

# Here, specify the 'directionality' of the alternative hypothesis. You have three choices:
# * "two.sided" (Note the period!) - Your alternative hypothesis is that the difference in averages is NOT EQUAL TO the number specified immediately above for the null hypothesis. This is also called a 'two-tailed' t-test.
# * "greater" - Your alternative hypothesis is that the difference in averages is GREATER THAN the number specified immediately above for the null hypothesis. This is one kind of 'one-tailed' t-test.
# * "less" - Your alternative hypothesis is that the difference in averages is LESS THAN the number specified immediately above for the null hypothesis. This is one kind of 'one-tailed' t-test.
#=====[5]=====
Directionality_AlternativeHypothesis = "two.sided"
#=============

# In the t-test, you can choose whether you want to assume that  variances of the two samples are equal. To determine whether this assumption is valid or not, the following calculates the variance of each of the two distributions for you.
#=====[6]=====
var(NumberDataA)
var(NumberDataB)
#=============

# Here, indicate whether you want to make the assumption that the variances are equal.
# If the two numbers calculated above are reasonably close to each other, then choose TRUE, and the variances from the two distribution will be pooled together.
# If the two numbers calculated above are not very close to each other, then choose FALSE, and the variance will be estimated separately for both groups. Moreover, the Welch-Satterthwaite approximation to the degrees of freedom will be used.
#=====[7]=====
VariancesEqual = TRUE
#=============

# In the output, it gives you a confidence interval. R defaults to a 95% confidence interval, as this is common practice in the literature, but you can adjust the percentage here if you like:
#=====[8]=====
PercentageForConfidenceInterval = 0.95
#=============

# Now run the actual 't.test()' function. 
# Note that R will issue an error message if the input data is unvarying/constant.
#=====[9]=====
Results = t.test( x=NumberDataA, y=NumberDataB, alternative=Directionality_AlternativeHypothesis, mu=DifferenceInAverages_NullHypothesis, paired=FALSE, var.equal=VariancesEqual, conf.level=PercentageForConfidenceInterval)
#=============

#######################
# INSPECT THE RESULTS #
#######################

# Run the following code for a reformatted, easy-to-read representation of the results:
#=====[10]=====
if(VariancesEqual==TRUE){VarianceAssumption="The analysis assumed that the two distributions each had the same variance.\n"}else{VarianceAssumption="The analysis assumed the two distributions have different variances.\n";Results$method="Two Sample t-test"};if(as.character(Results$alternative)=="two.sided"){AlternativeText=paste("The true difference in means is not equal to ", as.numeric(Results$null.value),".",sep="")}; if(as.character(Results$alternative)=="greater"){AlternativeText=paste("The true difference in means is greater than ", as.numeric(Results$null.value),".",sep="")}; if(as.character(Results$alternative)=="less"){AlternativeText=paste("The true difference in means is less than ", as.numeric(Results$null.value),".",sep="")}; cat("\n\nDESCRIPTIVE STATISTICS\n~~~~~~~~~~~~~~~~~~~~~~\nThe mean of " ,ColumnNameA," is ", as.numeric(Results$estimate)[1],".\nThe mean of ",ColumnNameB," is ",as.numeric(Results$estimate)[2],".\nHence the actual calculated difference in means\n   (",ColumnNameA," minus ", ColumnNameB,") is ", as.numeric(Results$estimate[1] - Results$estimate[2]),".\n\nTEST THAT WAS RUN\n~~~~~~~~~~~~~~~~~\nAn Unpaired ",Results$method, " was performed on ", ColumnNameA," and ", ColumnNameB,".\n", " - Null hypothesis: The true difference in means is ", as.numeric(Results$null.value),".\n - Alternative hypothesis: ",AlternativeText,"\n",VarianceAssumption,"\nRESULTS\n~~~~~~~\nThe ",100*PercentageForConfidenceInterval,"% confidence interval for the difference in means ranges\n   from ", as.numeric(Results$conf.int[1]), " to ", as.numeric(Results$conf.int[2]),".\nt(", as.numeric(Results$parameter), ")=", as.numeric(Results$statistic), ", p=",as.numeric(Results$p.value),"\n\n\n",sep="")
#=============

# Here is R's default display of the results:
#=====[11]=====
show(Results)
#=============
